﻿//css_ref System.Data;
//css_ref System.Xml;
//css_ref Interop.SAPbobsCOM.dll;
//css_ref log4net.dll;
//css_ref Produmex.Foundation.Data.dll;
//css_ref Produmex.Foundation.Data.Sbo.dll;
//css_ref Produmex.Foundation.dll;
//css_ref Produmex.Foundation.Sbo.dll;
//css_ref Produmex.Sbo.Logex.Data.dll;

using System;
using System.Text;
using System.Transactions;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.Attributes;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.SboGui;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.BusinessObjects;

namespace AddUserFieldsAndUserQueries
{
    class Program
    {
        const string CONNECTION_STRING = "Server=localhost;CompanyDB=SBODemo_US;ExtraDb=PMX_SBODemo_US;UseTrusted=False;DbUserName=sa;DbPassword=B1Admin;UserName=manager;Password=manager;Pooling=True;MinPoolSize=2;MaxPoolSize=10;ServerType=MSSQL2005;DBCultureInfo=en-US;LicenseServer=localhost:30000";

        private static class PmxUserQueryName
        {
            public const string VIYO_GET_BP_PRODUCTION_REMARKS = "VIYO_GET_BP_PRODUCTION_REMARKS";
        }

        private static class Defs
        {
            public const string PRODUCTION_ORDER_CARDCODE_ID = "18";  // Why not 68?
            public const string PRODUCTION_ORDER_REMARKS_ID = "3";
        }

        static void Main( string[] args )
        {
            try
            {
                using ( SboConnection conn = new SboConnection( CONNECTION_STRING ) )
                {
                    conn.Open(); //Open connection to SAP
                    Console.WriteLine( "Connection is open to database" );

                    // * UDF's *
                    SboUserFieldMDProvider udfProv = new SboUserFieldMDProvider( conn );

                    SboColumnDefinitionAttribute colDef = new SboColumnDefinitionAttribute( "U_PMX_VIYO_PROD_REM", "Productieopmerkingen", SboFieldType.Memo );
                    udfProv.AddUpdateUserfield( PmxBusinessPartnerTable.NAME, colDef );
                    colDef = new SboColumnDefinitionAttribute( "U_PMX_VIYO_PROD_QTY", "Productiehoeveelheid", SboFieldType.Float, -1, SboFieldSubType.Quantity );
                    udfProv.AddUpdateUserfield( PmxItemTable.NAME, colDef );

                    using ( TransactionScope scope = PmxDbConnection.GetNewTransactionScope() )
                    {
                        conn.EnlistTransaction( Transaction.Current );

                        // * formatted search *
                        PmxDbVersionUpdaterProvider dbUpdProv = new PmxDbVersionUpdaterProvider( conn );
                        SboUserQueryCategoryProvider uqcProv = new SboUserQueryCategoryProvider( conn );
                        SboUserQueryProvider uqProv = new SboUserQueryProvider( conn );

                        int categoryCode;
                        using ( SboUserQueryCategory category = uqcProv.GetUserQueryCategoryByName( "Viyo" ) )
                        {
                            if ( category != null )
                            {
                                categoryCode = category.Code;
                            }
                            else
                            {
                                using ( SboUserQueryCategory newCatg = uqcProv.GetNewBO() )
                                {
                                    newCatg.Name = "Viyo";
                                    newCatg.Permissions = "YYYYYYYYYYYYYYYYYYYY";

                                    categoryCode = Convert.ToInt32( uqcProv.AddBO( newCatg ) );
                                }
                            }
                        }

                        int userQueryID = dbUpdProv.AddUpdateUserQuery( PmxUserQueryName.VIYO_GET_BP_PRODUCTION_REMARKS, categoryCode, GetUserQueryGetBPProductionRemarks( PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CardCode.NAME ) );

                        // Add formatted searches using user query PMX_GET_BP_PRODUCTION_REMARKS
                        dbUpdProv.AddUpdateFormattedSearch(
                            SboFormType.PRODUCTION_ORDER,
                            Defs.PRODUCTION_ORDER_REMARKS_ID,
                            "-1",
                            userQueryID,
                            Defs.PRODUCTION_ORDER_CARDCODE_ID,
                            false );

                        PmxReportProvider repProv = new PmxReportProvider( conn );
                        PmxReport report = repProv.GetReportByPath( "Viyo_ProdOrderWorkNotes.rpt" );
                        if ( report == null )
                        {
                            report = repProv.GetNewBO();
                            report.Name = "Viyo_ProdOrderWorkNotes.rpt";
                            report.Orientation = PmxOrientation.Portrait;
                            report.PageSizeCode = "A4";
                            report.Path = "Viyo_ProdOrderWorkNotes.rpt";
                            report.ReportFormat = PmxReportFormat.CrystalReports;
                            report.ReportTypeCode = PmxReportTypeCodes.CUSTOM_REPORT_TYPE_CODE;
                            report.UseExtraDatabase = true;
                            repProv.AddBO( report );
                        }
                    }

                    //Complete transaction
                    scope.Complete();
                }
            }
            catch ( Exception ex )
            {
                Console.WriteLine( ex );
            }
        }


        /// <summary>
        /// Gets the user query get BP production remarks.
        /// </summary>
        /// <param name="refTable">The ref table.</param>
        /// <param name="refColumn">The ref column.</param>
        /// <returns></returns>
        internal static string GetUserQueryGetBPProductionRemarks( string refTable, string refColumn )
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.AppendFormat( "SELECT {0}",
                SqlCommandHelper.EscapeTableColumnName( PmxBusinessPartnerTable.NAME, "U_PMX_VIYO_PROD_REM" ) );
            strBuilder.AppendFormat( "\r\n FROM {0}", SqlCommandHelper.EscapeTableName( PmxBusinessPartnerTable.NAME ) );
            strBuilder.AppendFormat( "\r\n WHERE {0} = $[{1}.{2}]",
                SqlCommandHelper.EscapeTableColumnName( PmxBusinessPartnerTable.NAME, PmxBusinessPartnerTable.Columns.CardCode.NAME ),
                refTable, refColumn );

            return strBuilder.ToString();
        }
    
    
    }
}
